package com.shanp.sms;

import com.shanp.base.rx.RxBus;
import com.shanp.base.rx.RxSchedulerHelper;
import com.shanp.sms.model.SmsRespones;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import io.reactivex.Flowable;
import io.reactivex.Observable;
import io.reactivex.ObservableEmitter;
import io.reactivex.ObservableOnSubscribe;
import io.reactivex.ObservableTransformer;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;

/**
 * shanpin_lxw
 * 2019/3/20
 */
class SmsRepository {

    private static volatile SmsRepository INSTANCE = null;

    private SmsRepository() {
    }

    public static SmsRepository getInstance() {
        if (INSTANCE == null) {
            synchronized (SmsRepository.class) {
                if (INSTANCE == null) {
                    INSTANCE = new SmsRepository();
                }
            }
        }
        return INSTANCE;
    }

    private final static String U_GET_CONFIG = "/api/sms/access_ticket/v1";
    private final static String U_GET_IMG = "/api/sms/img_code/v1";
    private final static String U_SEND = "/api/sms/send/v1";

    private static final int GET_VERIFY_CODE_INTERVAL_TIME = 60;
    private volatile int mVerifyCodeTime;
    private String mTicketNo;

    private Disposable verifyCodeCountDown() {
        return Flowable.interval(0, 1, TimeUnit.SECONDS)
                .take(GET_VERIFY_CODE_INTERVAL_TIME + 1)
                .subscribe(new Consumer<Long>() {
                    @Override
                    public void accept(Long along) throws Exception {
                        int i = along.intValue();
                        mVerifyCodeTime = GET_VERIFY_CODE_INTERVAL_TIME - i;
                        CodeTimeEvent event = new CodeTimeEvent(mVerifyCodeTime);
                        RxBus.getInstance().post(event);
                    }
                });
    }

    private <T> ObservableTransformer<T, T> io_main() {
        return RxSchedulerHelper.io_main();
    }

    public String getTicketNo(){
        return mTicketNo;
    }

    Observable<SmsModel> getSmsConfig(final String phone) {
        return Observable.create(new ObservableOnSubscribe<SmsModel>() {
            @Override
            public void subscribe(ObservableEmitter<SmsModel> e) throws Exception {
                Map<String, Object> params = new HashMap<>(1);
                params.put("busiCode", SmsConfig.BUSINESS_CODE);
                params.put("phone", phone);
                SmsRespones response = SmsNetClient.get(SmsConfig.HOST + U_GET_CONFIG, params);
                if (response.getCode() == SmsNetClient.CODE_SUCCESS) {
                    SmsModel config = response.getDataToObj(SmsModel.class);
                    mTicketNo = config.getTicketNo();
                    e.onNext(config);
                    e.onComplete();
                } else {
                    e.onError(new Throwable(response.getCode() + ":" + response.getMsg()));
                }
            }
        }).compose(this.<SmsModel>io_main());
    }

    Observable<String> getSmsImg(final String ticketNo, final String path) {
        return Observable.create(new ObservableOnSubscribe<String>() {
            @Override
            public void subscribe(ObservableEmitter<String> e) throws Exception {
                //本地无缓存，从网络获取
                String url = SmsConfig.HOST + U_GET_IMG;
                Map<String, Object> map = new HashMap<>();
                map.put("busiCode", SmsConfig.BUSINESS_CODE);
                map.put("ticketNo", ticketNo);
                InputStream input = SmsNetClient.download(url, map);
                if (input == null) {
                    e.onError(new Throwable("download fail"));
                } else {
                    if (save(input, path)) {
                        //下载保存成功
                        e.onNext(path);
                        e.onComplete();
                    } else {
                        e.onError(new Throwable("download fail"));
                    }
                }
            }
        }).compose(this.<String>io_main());
    }

    public Observable<String> send(final String ticketNo, String sign, String imgCode) {
        final Map<String, Object> params = new HashMap<>();
        params.put("busiCode", SmsConfig.BUSINESS_CODE);
        params.put("ticketNo", ticketNo);
        params.put("sign", sign);
        params.put("imgCode", imgCode);
        return Observable.create(new ObservableOnSubscribe<String>() {
            @Override
            public void subscribe(ObservableEmitter<String> e) throws Exception {
                SmsRespones response = SmsNetClient.post(SmsConfig.HOST + U_SEND, params);
                if (response.getCode() == SmsNetClient.CODE_SUCCESS) {
                    verifyCodeCountDown();
                    e.onNext("");
                    e.onComplete();
                } else {
                    e.onError(new Throwable(response.getCode() + ":" + response.getMsg()));
                }
            }
        }).compose(this.<String>io_main());
    }

    public boolean save(InputStream input, String path) {
        FileOutputStream output = null;
        try {
            output = new FileOutputStream(new File(path));
            byte[] buffer = new byte[1024];
            int index;
            while ((index = input.read(buffer)) > 0) {
                output.write(buffer, 0, index);
            }
        } catch (Exception e) {
            return false;
        } finally {
            try {
                if (input != null) {
                    input.close();
                }
                if (output != null) {
                    output.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return true;
    }
}
