package com.shanp.sms.net;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

/**
 * Leexw
 * 17/1/9
 */
public class DefaultNetClientImpl implements NetClient {

    /**
     * 连接超时时间
     */
    private static final int CONNECT_TIMEOUT = 10000;
    /**
     * 上传数据超时时间
     */
    private static final int UPDATE_TIMEOUT = 60000;
    /**
     * 普通读取数据超时时间
     */
    private static final int READ_TIMEOUT = 30000;

    private static final String CHARSET = "UTF-8";
    private static final int SUCCESS_CODE = 200;

    private String mUserAgent;
    private Map<String, String> mPropertyManager;

    @Override
    public void setUserAgent(String agent) {
        mUserAgent = agent;
    }

    @Override
    public void setRequestProperty(String key, String value) {
        if (mPropertyManager == null) {
            mPropertyManager = new HashMap<>();
        }
        mPropertyManager.put(key, value);
    }

    private void setRequestProperty(HttpURLConnection connection) {
        if (mPropertyManager == null) {
            return;
        }
        Set<String> keySet = mPropertyManager.keySet();
        for (String key : keySet) {
            String value = mPropertyManager.get(key);
            connection.setRequestProperty(key, value);
        }
    }

    @Override
    public InputStream download(String url) {
        try {
            URL requestUrl = new URL(url);
            HttpURLConnection connection = (HttpURLConnection) requestUrl.openConnection();
            connection.setDoInput(true);
            connection.setUseCaches(false);
            connection.setConnectTimeout(CONNECT_TIMEOUT);
            connection.setReadTimeout(UPDATE_TIMEOUT);
            setRequestProperty(connection);
            connection.setRequestProperty("Accept-Charset", CHARSET);
            connection.connect();
            int code = connection.getResponseCode();
            if (code == SUCCESS_CODE) {
                return connection.getInputStream();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String get(String url) {
        return request(url, "GET");
    }

    @Override
    public String post(String url, String params) {
        return request(url, params, "POST");
    }

    @Override
    public String put(String url, String params) {
        return request(url, params, "PUT");
    }

    @Override
    public String delete(String url, String params) {
        url = url + "?" + params;
        return request(url, "DELETE");
    }

    public String request(String url, String method) {
        try {
            HttpURLConnection connection = generateConnection(url, method);
            connection.connect();
            int code = connection.getResponseCode();
            if (code == SUCCESS_CODE) {
                return getResultByInputStream(connection.getInputStream());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    public String request(String url, String param, String method) {
        OutputStream out = null;
        try {
            HttpURLConnection connection = generateConnection(url, method);
            connection.setDoOutput(true);
            connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            out = connection.getOutputStream();
            out.write(param.getBytes());
            out.flush();
            out.close();
            int code = connection.getResponseCode();
            if (code == SUCCESS_CODE) {
                return getResultByInputStream(connection.getInputStream());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    private HttpURLConnection generateConnection(String url) throws Exception {
        return generateConnection(url, "GET");
    }

    private HttpURLConnection generateConnection(String url, String method) throws Exception {
        URL requestUrl = new URL(url);
        HttpURLConnection connection = (HttpURLConnection) requestUrl.openConnection();
        connection.setDoInput(true);
        connection.setUseCaches(false);
        connection.setConnectTimeout(CONNECT_TIMEOUT);
        connection.setReadTimeout(READ_TIMEOUT);
        connection.setRequestProperty("Accept-Charset", CHARSET);
        connection.setRequestMethod(method);
        setRequestProperty(connection);
        return connection;
    }

    private String getResultByInputStream(InputStream stream) {
        BufferedReader buffered = null;
        try {
            buffered = new BufferedReader(new InputStreamReader(stream));
            StringBuilder sb = new StringBuilder();
            String line;
            while ((line = buffered.readLine()) != null) {
                sb.append(line);
            }
            buffered.close();
            return sb.toString();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (buffered != null) {
                try {
                    buffered.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    @Override
    public String upload(String url, Map<String, Object> params, List<File> files) {
        String BOUNDARY = UUID.randomUUID().toString(); // 边界标识 随机生成
        String PREFIX = "--", LINE_END = "\r\n";
        String CONTENT_TYPE = "multipart/form-data"; // 内容类型
        try {
            URL requestUrl = new URL(url);
            HttpURLConnection connection = (HttpURLConnection) requestUrl.openConnection();
            connection.setReadTimeout(UPDATE_TIMEOUT);
            connection.setConnectTimeout(UPDATE_TIMEOUT);
            connection.setDoInput(true); // 允许输入流
            connection.setDoOutput(true); // 允许输出流
            connection.setUseCaches(false); // 不允许使用缓存
            connection.setRequestMethod("POST"); // 请求方式
            setRequestProperty(connection);
            connection.setRequestProperty("Charset", CHARSET); // 设置编码
            connection.setRequestProperty("connection", "keep-alive");
            connection.setRequestProperty("Content-Type", CONTENT_TYPE + ";boundary="
                    + BOUNDARY);
            //当文件不为空，把文件包装并且上传
            OutputStream outputSteam = connection.getOutputStream();
            DataOutputStream dos = new DataOutputStream(outputSteam);
            StringBuilder sb = new StringBuilder();
            String uploadParamName = "upload";
            if (params != null) {
                for (Map.Entry<String, Object> entry : params.entrySet()) {
                    Object value = entry.getValue();
                    if (value != null) {
                        //设置POST参数
                        String key = entry.getKey();
                        if ("uploadParamName".equals(key)) {
                            uploadParamName = value.toString();
                            continue;
                        }
                        sb.append(PREFIX).append(BOUNDARY).append(LINE_END);
                        sb.append("Content-Disposition: form-data; name=\"")
                                .append(key)
                                .append("\"")
                                .append(LINE_END);
                        sb.append(LINE_END);
                        sb.append(value.toString());
                        sb.append(LINE_END);
                    }
                }
            }
            dos.write(sb.toString().getBytes(CHARSET));
            int size = files.size();
            for (int i = 0; i < size; i++) {
                final File file = files.get(i);
                sb.delete(0, sb.length());
                //分割符
                sb.append(PREFIX).append(BOUNDARY).append(LINE_END);
                //注意： name里面的值为服务器端需要key 只有这个key 才可以得到对应的文件
                //filename是文件的名字，包含后缀名的 比如:abc.png
                sb.append("Content-Disposition: form-data; name=\"").append(uploadParamName);
                sb.append(i > 0 ? i : "");
                sb.append("\"; filename=\"")
                        .append(file.getName())
                        .append("\"")
                        .append(LINE_END);
                sb.append("Content-Type: application/octet-stream; charset=" + CHARSET)
                        .append(LINE_END)
                        .append(LINE_END);
                dos.write(sb.toString().getBytes(CHARSET));
                InputStream is = new FileInputStream(file);
                byte[] bytes = new byte[1024];
                int len;
                while ((len = is.read(bytes)) != -1) {
                    dos.write(bytes, 0, len);
                }
                is.close();
                dos.write(LINE_END.getBytes());
            }
            byte[] end_data = (PREFIX + BOUNDARY + PREFIX + LINE_END)
                    .getBytes();
            dos.write(end_data);
            dos.flush();
            //获取上传文件响应
            int code = connection.getResponseCode();
            if (code == 200) {
                BufferedReader buffread = new BufferedReader(new InputStreamReader(connection.getInputStream()));
                StringBuilder reSb = new StringBuilder();
                String line;
                while ((line = buffread.readLine()) != null) {
                    reSb.append(line);
                }
                buffread.close();
                return reSb.toString();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}
