package com.shanp.base.net;

import android.util.Log;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.net.Proxy;
import java.util.concurrent.TimeUnit;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLSession;

import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;

public class NetManager {

    private static final int DEFAULT_READ_TIME_OUT = 15;
    private static final int DEFAULT_WRITE_TIME_OUT = 20;
    private static final int DEFAULT_CONNECT_TIME_OUT = 15;

    private Retrofit mRetrofit;

    private NetManager(boolean debug, String server, Interceptor header) {
        OkHttpClient.Builder mBuilder = new OkHttpClient.Builder();
        mBuilder.proxy(Proxy.NO_PROXY);
        //https配置
        HttpsUtil.SSLParams params = HttpsUtil.getSSLParams();
        if (params != null) {
            mBuilder.sslSocketFactory(params.sslSocketFactory, params.trustManager);
            mBuilder.hostnameVerifier(new HostnameVerifier() {
                @Override
                public boolean verify(String hostname, SSLSession session) {
                    return true;
                }
            });
        }
        //日志拦截器配置
        if (debug) {
            HttpLoggingInterceptor loggingInterceptor = new HttpLoggingInterceptor(new HttpLoggingInterceptor.Logger() {
                @Override
                public void log(String message) {
                    Log.d("SHANP-HTTP", message);
                }
            })
                    .setLevel(HttpLoggingInterceptor.Level.BODY);
            mBuilder.addInterceptor(loggingInterceptor);
        }
        //头部拦截器配置
        if (header != null) {
            mBuilder.addInterceptor(header);
        }

        mBuilder.readTimeout(DEFAULT_READ_TIME_OUT, TimeUnit.SECONDS)
                .writeTimeout(DEFAULT_WRITE_TIME_OUT, TimeUnit.SECONDS)
                .connectTimeout(DEFAULT_CONNECT_TIME_OUT, TimeUnit.SECONDS);

        OkHttpClient mOkHttpClient = mBuilder.build();

        Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").serializeNulls().create();

        mRetrofit = new Retrofit.Builder()
                .baseUrl(server)
                .client(mOkHttpClient)
                .addConverterFactory(GsonConverterFactory.create(gson))
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                .build();
    }

    <T> T create(Class<T> service) {
        return mRetrofit.create(service);
    }

    public final static class Builder {
        private boolean mDebug;
        private String mServer;
        private Interceptor mHeaderInterceptor;

        private NetManager mNetManager;

        public Builder debug(boolean debug) {
            this.mDebug = debug;
            return this;
        }

        public Builder server(String server) {
            this.mServer = server;
            return this;
        }

        public Builder header(Interceptor header) {
            mHeaderInterceptor = header;
            return this;
        }

        public void build() {
            mNetManager = new NetManager(mDebug, mServer, mHeaderInterceptor);
        }

        NetManager getNetManager() {
            return mNetManager;
        }
    }
}
